/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include "idt82V3910.h"
#include "profiles.h"
#include "inputFreq.h"
#include "outputFreq.h"

/**
 * @brief Sample configuration 1
 *
 *      IN3 (25 MHz) -> T0 +-> OUT1 (25 MHz)
 *                         +-> OUT6 (156.25 MHz)
 *
 * @param hdlr Device driver handler
 */
void IDTSample_ConfigureSampleConfig1(T_idtDrvHdlr hdlr)
{
        /* Disable all outputs */
        IDTHlApi_DisableAllOutputs(hdlr);

        /* Configure input 3, 25MHz */
        IDTHlApi_ConfigureInputFrequency(hdlr, 3, 25000);

        /* Configure DPLL for G8262 Option 1 */
        IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_1);

        /* Configure output output 1 (25 MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        1,
                        OutFreq_25000k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX, /* Program the backup APLL configuration */
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX); /* We're only using one of the DPLLs */


        /* Configure output output 6 (156.25 MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        6,
                        OutFreq_156250k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX, /* Program the backup APLL configuration */
                        APLL_OUT_SIG_LVPECL,
                        APLL_CLK_SEL_INTERNAL); /* We're only using one of the DPLLs */

        /* Switch to backup APLL configuration now it's configured 
                * Note: this step is not required if you specify which configuration in
                * IDTHlApi_ConfigureOutput function, i.e. APLL_CONFIG0 or APLL_CONFIG1
                */
        IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_1);

        /* Enable output */
        IDTOutput_SetOutputEnable(hdlr, 1, 1);
        IDTOutput_SetOutputEnable(hdlr, 6, 1);
}

/**
 * @brief Sample configuration 2
 *
 *      IN3 (25 MHz) -> T0 +-> OUT1 (25 MHz)
 *                         +-> OUT3 (8 kHz)
 *                         +-> OUT5 (125 MHz)
 *
 * @param hdlr Device driver handler
 */
void IDTSample_ConfigureSampleConfig2(T_idtDrvHdlr hdlr)
{
        /* Disable all outputs */
        IDTHlApi_DisableAllOutputs(hdlr);

        /* Configure input 3, 25MHz */
        IDTHlApi_ConfigureInputFrequency(hdlr, 3, 25000);

        /* Configure DPLL for G8262 Option 1 */
        IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_1);

        /* Configure output output 1 (25 MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        1,
                        OutFreq_25000k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX, /* Program the backup APLL configuration */
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX); /* We're only using one of the DPLLs */

        /* Configure output output 3 (8 kHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        3,
                        OutFreq_8k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX, /* Program the backup APLL configuration */
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX); /* We're only using one of the DPLLs */

        /* Configure output output 5 (125 MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        5,
                        OutFreq_125000k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX, /* Program the backup APLL configuration */
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX); /* We're only using one of the DPLLs */

        /* Enable output */
        IDTOutput_SetOutputEnable(hdlr, 1, 1);
        IDTOutput_SetOutputEnable(hdlr, 3, 1);
        IDTOutput_SetOutputEnable(hdlr, 5, 1);
}

/**
 * @brief Sample configuration 3
 *
 *      IN3 (25 MHz) -> T0 +-> OUT1 (25 MHz)
 *                         +-> OUT3 (10 MHz)
 *                         +-> OUT5 (1 PPS)
 *
 * @param hdlr Device driver handler
 */
void IDTSample_ConfigureSampleConfig3(T_idtDrvHdlr hdlr)
{
        /* Disable all outputs */
        IDTHlApi_DisableAllOutputs(hdlr);

        /* Configure input 3, 25MHz */
        IDTHlApi_ConfigureInputFrequency(hdlr, 3, 25000);

        /* Configure DPLL for G8262 Option 1 */
        IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_1);

        /* Configure output output 1 (25 MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        1,
                        OutFreq_25000k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX, /* Program the backup APLL configuration */
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX); /* We're only using one of the DPLLs */

        /* For 1 PPS */
        {
                /* Disable hitless switching */
                IDTDpll_SetHitlessSwitchingFeature(hdlr, 0, 0);

                /* Enable 1 PPS fast lock only for phase */
                IDTDpll_SetPpsFastLock(hdlr, 0, 1);

                /* Phase slope unlimited */
                IDTDpll_SetPhaseSlope(hdlr, Dpll_T0, phaseSlope_noLimit);
        }

        /* Configure output output 3 (10 MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        3,
                        OutFreq_10000k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX, /* Program the backup APLL configuration */
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX); /* We're only using one of the DPLLs */

        /* Configure output output 5 (1 PPS) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        5,
                        OutFreq_1,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX, /* Program the backup APLL configuration */
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX); /* We're only using one of the DPLLs */
		  
        /* Enable output */
        IDTOutput_SetOutputEnable(hdlr, 1, 1);
        IDTOutput_SetOutputEnable(hdlr, 3, 1);
        IDTOutput_SetOutputEnable(hdlr, 5, 1);
}

/**
 * @brief Sample configuration 4
 *
 *      Inputs: IN3 (10MHz), IN4 (1.544MHz), IN5 (25MHz), IN6 (8KHz), IN7 (8KHz),
 *              IN8 (156.25MHz), IN9 (156.25MHz), IN10 (25MHz), IN11 (25MHz)
 *      Outputs: OUT1 (1.544MHz), OUT3 (10MHz), OUT4 (25MHz), OUT7 (156.25MHz),
 *               OUT10 (156.25MHz), OUT11 (156.25MHz)
 *      T0 Profile: G.8262 Option 2
 *      T4 Profile: Default
 *      XTAL1/2: 25MHz
 *
 * @param hdlr Device driver handler
 */
void IDTSample_ConfigureSampleConfig4(T_idtDrvHdlr hdlr)
{
        /* Disable all outputs */
        IDTHlApi_DisableAllOutputs(hdlr);

        /* Configure input 3, 10MHz */
        IDTHlApi_ConfigureInputFrequency(hdlr, 3, 10000);

        /* Configure input 4, 1.544MHz */
        IDTHlApi_ConfigureInputFrequency(hdlr, 4, 1544);

        /* Configure input 5, 25MHz */
        IDTHlApi_ConfigureInputFrequency(hdlr, 5, 25000);

        /* Configure input 6, 8KHz */
        IDTHlApi_ConfigureInputFrequency(hdlr, 6, 8);

        /* Configure input 7, 8KHz */
        IDTHlApi_ConfigureInputFrequency(hdlr, 7, 8);

        /* Configure input 8, 156.25MHz */
        IDTHlApi_ConfigureInputFrequency(hdlr, 8, 156250);

        /* Configure input 9, 156.25MHz */
        IDTHlApi_ConfigureInputFrequency(hdlr, 9, 156250);

        /* Configure input 10, 25MHz */
        IDTHlApi_ConfigureInputFrequency(hdlr, 10, 25000);

        /* Configure input 11, 25MHz */
        IDTHlApi_ConfigureInputFrequency(hdlr, 11, 25000);

        /* Configure DPLL for G8262 Option 2 */
        IDTHlApi_g8262EecOption2(hdlr, DPLL_INSTANCE_1);

        /* Configure output output 1 (1.544MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        1,
                        OutFreq_1544k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX,
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX);

        /* Configure output output 3 (10MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        3,
                        OutFreq_10000k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX,
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX);

        /* Configure output output 4 (25MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        4,
                        OutFreq_25000k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX,
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX);

        /* Configure output output 7 (156.25MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        7,
                        OutFreq_156250k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG0,
                        APLL_OUT_SIG_LVDS,
                        APLL_CLK_SEL_INTERNAL);

        /* Configure output 10 (156.25MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        10,
                        OutFreq_156250k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG0,
                        APLL_OUT_SIG_LVDS,
                        APLL_CLK_SEL_INTERNAL);

        /* Configure output 11 (156.25MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        11,
                        OutFreq_156250k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG0,
                        APLL_OUT_SIG_LVDS,
                        APLL_CLK_SEL_INTERNAL);

        /* Enable output */
        IDTOutput_SetOutputEnable(hdlr, 1, 1);
        IDTOutput_SetOutputEnable(hdlr, 3, 1);
        IDTOutput_SetOutputEnable(hdlr, 4, 1);
        IDTOutput_SetOutputEnable(hdlr, 7, 1);
        IDTOutput_SetOutputEnable(hdlr, 10, 1);
        IDTOutput_SetOutputEnable(hdlr, 11, 1);
}

/**
 * @brief Sample configuration 5
 *
 *      Carbon copy of IDTSample_ConfigureSampleConfig4,
 *      except there is no input and DPLL is in free-run
 *      mode (default).
 *
 * @param hdlr Device driver handler
 */
void IDTSample_ConfigureSampleConfig5(T_idtDrvHdlr hdlr)
{
        /* Disable all outputs */
        IDTHlApi_DisableAllOutputs(hdlr);

        /* Configure output output 1 (1.544MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        1,
                        OutFreq_1544k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX,
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX);

        /* Configure output output 3 (10MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        3,
                        OutFreq_10000k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX,
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX);

        /* Configure output output 4 (25MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        4,
                        OutFreq_25000k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX,
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX);

        /* Configure output output 7 (156.25MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        7,
                        OutFreq_156250k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG0,
                        APLL_OUT_SIG_LVDS,
                        APLL_CLK_SEL_INTERNAL);

        /* Configure output 10 (156.25MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        10,
                        OutFreq_156250k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG0,
                        APLL_OUT_SIG_LVDS,
                        APLL_CLK_SEL_INTERNAL);

        /* Configure output 11 (156.25MHz) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        11,
                        OutFreq_156250k,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG0,
                        APLL_OUT_SIG_LVDS,
                        APLL_CLK_SEL_INTERNAL);

        /* Enable output */
        IDTOutput_SetOutputEnable(hdlr, 1, 1);
        IDTOutput_SetOutputEnable(hdlr, 3, 1);
        IDTOutput_SetOutputEnable(hdlr, 4, 1);
        IDTOutput_SetOutputEnable(hdlr, 7, 1);
        IDTOutput_SetOutputEnable(hdlr, 10, 1);
        IDTOutput_SetOutputEnable(hdlr, 11, 1);
}

/**
 * @brief Sample configuration 1PPS
 *
 *      IN3 (1PPS) -> T0 -> OUT3 (1PPS)
 *
 * @param hdlr Device driver handler
 */
void IDTSample_Configure1Pps(T_idtDrvHdlr hdlr)
{
        /* Disable all outputs */
        IDTHlApi_DisableAllOutputs(hdlr);

        /* Configure input 3, 1 PPS (1 Hz) */
        IDTHlApi_ConfigureInput1PPS(hdlr, 3);

        /* Configure DPLL for 1 PPS */
        IDTHlApi_pps(hdlr, DPLL_INSTANCE_1);

        /* Configure output output 3 (1 PPS) using T0 */
        IDTHlApi_ConfigureOutput(hdlr,
                        3,
                        OutFreq_1,
                        DPLL_INSTANCE_1,
                        sonetGigeMode_Matching,
                        APLL_CONFIG_MAX, /* Program the backup APLL configuration */
                        APLL_OUT_SIG_MAX,
                        APLL_CLK_SEL_MAX); /* We're only using one of the DPLLs */

        /* Enable output */
        IDTOutput_SetOutputEnable(hdlr, 3, 1);
}

/**
 * @brief Sample configuration for 1PPS holdover issue
 * @param hdlr Device driver handler
 */
void IDTSample_Configure1PpsHoldover(T_idtDrvHdlr hdlr)
{
        int timeout = 20; /* Up to 20 seconds */
        int i;
        T_osUint8 status;

        /* Disable 1 PPS fast lock after we have locked to 1 PPS signal.
         */
        for (i=0; i<timeout; i++) {

                /* Is DPLL locked to input? */
                if ((status = IDTDpll_IsDpllLocked(hdlr, Dpll_T0)) == 1) {
                        OS_MSEC_DELAY(1);

                        /* Disable 1 PPS fast phase lock */
                        IDTDpll_SetPpsFastLock(hdlr, 0, 0);
                        return;
                }

                OS_MSEC_DELAY(1); /* Wait a second */
        }

        printf("Timed OUT!!!!\n");
}


/**
 * @brief Function to configure device for DCO mode 
 * @param hdlr Device driver handler
 *
 */
void IDTSample_ConfigureDcoMode(T_idtDrvHdlr hdlr)
{
        IDTDpll_SetDpllOperMod(hdlr, Dpll_T0, Dco_Mode);
        IDTDpll_SetPhaseSlope(hdlr, Dpll_T0, phaseSlope_noLimit);
}

/**
 * @brief Function to set combined DCO offset value. It uses both DCO
 * (holdover) and nominal offsets.
 * @param hdlr Device driver handler
 * @param pptOffset Parts per trillion offset
 */
void IDTSample_SetCombinedDcoOffset(T_idtDrvHdlr hdlr,
                          long      pptOffset)
{
        long remainderOffset = pptOffset;

        /* Try DCO first. Check if within DCO range */
        if ((pptOffset<=DCO_MAX_PPT) && (pptOffset >= DCO_MIN_PPT)) {
                /* Excellent! Covered by DCO range. */
                IDTDpll_SetHoldoverFreq(hdlr, Dpll_T0, pptOffset);
                IDTGeneral_SetOscFreqOffset(hdlr,0);
                return;
        }

        /* Boo! Now we have to use nominal frequency offset in addition to
           DCO offset */
        if (pptOffset>DCO_MAX_PPT) {

                /* Apply maximum DCO offset (92274677 ppt) */
                remainderOffset -= DCO_MAX_PPT;
                IDTDpll_SetHoldoverFreq(hdlr, Dpll_T0, DCO_MAX_PPT);
        }
        if (pptOffset<DCO_MIN_PPT) {

                /* Apply minimum DCO offset (-92274688 ppt) */
                remainderOffset -= DCO_MIN_PPT;
                IDTDpll_SetHoldoverFreq(hdlr, Dpll_T0, DCO_MIN_PPT);
        }

        /* Limit offset */
        if (remainderOffset > NOMINAL_MAX_PPT)
                remainderOffset = NOMINAL_MAX_PPT;
        if (remainderOffset < NOMINAL_MIN_PPT)
                remainderOffset = NOMINAL_MIN_PPT;

        /* For nominal frequency offset:
         *
         *   REGVAL = (-offset * 10) / 844
         *
         *      where offset is in parts per trillion (PPT)
         *            REGVAL is 24 bit 2's compliment
         */
        IDTGeneral_SetOscFreqOffset(hdlr,remainderOffset);
}

/**
 * @brief Function to get combined DCO offset value. It uses both DCO
 * (holdover) and nominal offsets.
 * @param hdlr Device driver handler
 * @return Parts per trillion offset
 */
long IDTSample_GetCombinedDcoOffset(T_idtDrvHdlr hdlr)
{
        return ( IDTDpll_GetHoldoverFreq(hdlr, Dpll_T0) +
                 IDTGeneral_GetOscFreqOffset(hdlr));
}
